﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace SL_DG_Flex_PartialTextForeGround
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            #region "Data"

            var list = new List<TestData>();
            list.Add(new TestData() { Name = "Jason Bourne", Title = "Mr." });
            list.Add(new TestData() { Name = "Marie St.Jacques", Title = "Mrs." });
            list.Add(new TestData() { Name = "Alexander Conklin", Title = "Mr." });
            list.Add(new TestData() { Name = "David Webb", Title = "Mr." });
            list.Add(new TestData() { Name = "Gordon Webb", Title = "Mr." });
            list.Add(new TestData() { Name = "Joshua Webb", Title = "Master" });

            #endregion

            c1FlexGrid1.ItemsSource = list;

            string _text = string.Empty;

            //Search at "Find" button click
            button1.Click += (s, e) =>
            {
                _text = this.textBox1.Text;
                //refresh grid each time button is clicked for a new search
                c1FlexGrid1.CellFactory = new MyCellFactory(_text);
            };

            //Search at "Enter" keydown
            textBox1.KeyDown += (s, e) =>
                {
                    if (e.Key == Key.Enter)
                    {
                        _text = this.textBox1.Text;
                        c1FlexGrid1.CellFactory = new MyCellFactory(_text);
                    }
                };
        }
    }

    //Customized CellFactory
    public class MyCellFactory : C1.Silverlight.FlexGrid.CellFactory
    {
        private string _text;

        public MyCellFactory()
        { }

        public MyCellFactory(string searchString)
        {
            _text = searchString;
        }

        public override void CreateCellContent(C1.Silverlight.FlexGrid.C1FlexGrid grid, Border bdr, C1.Silverlight.FlexGrid.CellRange range)
        {
            base.CreateCellContent(grid, bdr, range);

            var tb = bdr.Child as TextBlock;
            if (tb != null)
                HighlightCell(tb);
        }

        private void HighlightCell(TextBlock tb)
        {
            if (string.Empty != _text)
            {
                var stringContent = tb.Text;
                if (stringContent.ToLower().Contains(_text.ToLower()))
                {
                    var runsToHighlight = new List<Run>();
                    var newInlines = BuildRuns(stringContent, _text, runsToHighlight);

                    // apply new splitted InlineCollection
                    tb.Inlines.Clear();
                    foreach (var il in newInlines) tb.Inlines.Add(il);

                    // apply styles:
                    // red color to the matching text
                    // Bold FontWeight too
                    foreach (var r in runsToHighlight)
                    {
                        r.Foreground = new SolidColorBrush(Colors.Red);
                        r.FontWeight = FontWeights.Bold;
                    }
                }
            }
        }

        private List<Run> BuildRuns(string text, string filterText, List<Run> runsToHighlight)
        {
            var runs = new List<Run>();
            int i = 0;

            while (i < text.Length)
            {
                var pos = text.IndexOf(filterText, i, StringComparison.CurrentCultureIgnoreCase);
                // not matching text in the head
                if (i < pos)
                    runs.Add(new Run() { Text = text.Substring(i, pos - i) });

                if (pos == -1)
                {
                    // not matching text in the tail
                    runs.Add(new Run() { Text = text.Substring(i) });
                    i = text.Length;
                }
                else
                {
                    // new matching text
                    var hRun = new Run() { Text = text.Substring(pos, filterText.Length) };
                    runs.Add(hRun);
                    runsToHighlight.Add(hRun);
                    i = pos + filterText.Length;
                }
            }
            return runs;
        }
    }

    public class TestData
    {
        public string Name { get; set; }
        public string Title { get; set; }
    }

}
